#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <getopt.h>
#include <unistd.h>
#include <netdb.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>


static int __client(int sk)
{
	printf("Connected to the host\n");
	while (1) {
		char buffer[256];
		const int count = read(0, buffer, sizeof(buffer));

		if (!count)
			break;

		for (int i = 0; i != count;) {
			const int written = write(sk, buffer + i, count - i);

			if (written < 0) {
				perror("Failed to send data to the host");
				return -1;
			}
			i += written;
		}
	}
	return 0;
}

static int client(const struct sockaddr *addr, socklen_t len)
{
	const int sk = socket(addr->sa_family, SOCK_STREAM, 0);

	if (sk < 0) {
		perror("Failed to create socket");
		return -1;
	}

	if (connect(sk, addr, len) < 0) {
		close(sk);
		perror("Failed to connect to the host");
		return -1;
	}

	const int rc = __client(sk);

	close(sk);
	return rc;
}

int main(int argc, char **argv)
{
	static const struct option long_opts[] = {
		{"host", required_argument, NULL, 'h'},
		{"port", required_argument, NULL, 'p'},
		{NULL, 0, NULL, 0}
	};
	static const char *optstr = "h:p:";

	const char *host = NULL;
	const char *port = "13131";
	struct addrinfo hint;
	struct addrinfo *info;
	int rc;

	while (1) {
		const int c = getopt_long(argc, argv, optstr, long_opts, NULL);
		char *endptr;

		if (c == -1)
			break;

		switch (c) {
		case 'p':
			port = optarg;
			break;
		case 'h':
			host = optarg;
			break;
		default:
			fprintf(stderr, "Unrecognized option %s\n",
						argv[optind]);
			exit(-1);
			break;
		}
	}

	if (!host) {
		fprintf(stderr, "Server address required\n");
		exit(-1);
	}

	memset(&hint, 0, sizeof(hint));
	hint.ai_family = AF_UNSPEC;
	hint.ai_socktype = SOCK_STREAM;

	if (getaddrinfo(host, port, &hint, &info)) {
		fprintf(stderr, "Failed to get addrinfo");
		exit(-1);
	}

	rc = -1;
	for (struct addrinfo *p = info; rc && p != NULL; p = p->ai_next)
		rc = client(p->ai_addr, p->ai_addrlen);

	freeaddrinfo(info);
	return rc;
}
