#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <pthread.h>
#include <getopt.h>
#include <unistd.h>
#include <netdb.h>

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>


static void handle_client(int sk)
{
	printf("Incoming connection\n");
	while (1) {
		char buffer[256];
		const int count = read(sk, buffer, sizeof(buffer));

		if (count < 0) {
			perror("Failed to read from client");
			return;
		}

		if (!count) {
			printf("Client closed conection\n");
			break;
		}
		write(1, buffer, count);
	}
}

static int __server(int ssk)
{
	while (1) {
		struct sockaddr_storage client_addr;
		socklen_t addr_len;

		const int csk = accept(ssk, (struct sockaddr *)&client_addr,
					&addr_len);

		if (csk < 0) {
			perror("Failed to accept incomming connection");
			return -1;
		}
		handle_client(csk);
		close(csk);
	}
	return 0;
}

static int server(const struct sockaddr *addr, socklen_t len)
{
	const int sk = socket(addr->sa_family, SOCK_STREAM, 0);

	if (sk < 0) {
		perror("Failed to create socket");
		return -1;
	}

	if (bind(sk, addr, len) < 0) {
		close(sk);
		perror("Failed to bind socket to the specified port");
		return -1;
	}

	if (listen(sk, 0) < 0) {
		close(sk);
		perror("Failed to start listening on the soscket");
		return -1;
	}

	const int rc = __server(sk);

	close(sk);
	return rc;
}

struct addr_info {
	struct sockaddr *addr;
	socklen_t len;
};

static void *server_thread(void *arg)
{
	struct addr_info *addr = arg;

	server(addr->addr, addr->len);
	free(addr);
	return NULL;
}

int main(int argc, char **argv)
{
	static const struct option long_opts[] = {
		{"port", required_argument, NULL, 'p'},
		{NULL, 0, NULL, 0}
	};
	static const char *optstr = "p:";

	const char *port = "13131";

	while (1) {
		const int c = getopt_long(argc, argv, optstr, long_opts, NULL);

		if (c == -1)
			break;

		switch (c) {
		case 'p':
			port = optarg;
			break;
		default:
			fprintf(stderr, "Unrecognized option %s\n",
						argv[optind]);
			exit(-1);
			break;
		}
	}

	struct addrinfo hint;
	struct addrinfo *info;

	memset(&hint, 0, sizeof(hint));
	hint.ai_family = AF_UNSPEC;
	hint.ai_socktype = SOCK_STREAM;

	if (getaddrinfo(NULL, port, &hint, &info)) {
		fprintf(stderr, "Failed to get addinfo");
		exit(-1);
	}

	for (struct addrinfo *p = info; p != NULL; p = p->ai_next) {
		struct addr_info *addr = malloc(sizeof(*addr) + p->ai_addrlen);
		pthread_t thread;

		assert(addr);
		addr->addr = (struct sockaddr *)(addr + 1);
		addr->len = p->ai_addrlen;
		memcpy(addr->addr, p->ai_addr, p->ai_addrlen);

		assert(pthread_create(&thread, NULL, server_thread, addr) == 0);
		assert(pthread_detach(thread) == 0);
	}

	freeaddrinfo(info);
	pthread_exit(NULL);
	return 0;
}
