#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <getopt.h>
#include <unistd.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>


static int __client(int sk)
{
	printf("Connected to the host\n");
	while (1) {
		char buffer[256];
		const int count = read(0, buffer, sizeof(buffer));

		if (!count)
			break;

		for (int i = 0; i != count;) {
			const int written = write(sk, buffer + i, count - i);

			if (written < 0) {
				perror("Failed to send data to the host");
				return -1;
			}
			i += written;
		}
	}
	return 0;
}

static int client(const struct sockaddr_in *addr)
{
	const int sk = socket(AF_INET, SOCK_STREAM, 0);

	if (sk < 0) {
		perror("Failed to create socket");
		return -1;
	}

	if (connect(sk, (struct sockaddr *)addr, sizeof(*addr)) < 0) {
		close(sk);
		perror("Failed to connect to the host");
		return -1;
	}

	const int rc = __client(sk);

	close(sk);
	return rc;
}

int main(int argc, char **argv)
{
	static const struct option long_opts[] = {
		{"host", required_argument, NULL, 'h'},
		{"port", required_argument, NULL, 'p'},
		{NULL, 0, NULL, 0}
	};
	static const char *optstr = "h:p:";

	struct sockaddr_in addr;
	const char *host = NULL;
	int port = 13131;

	while (1) {
		const int c = getopt_long(argc, argv, optstr, long_opts, NULL);
		char *endptr;

		if (c == -1)
			break;

		switch (c) {
		case 'p':
			port = strtoul(optarg, &endptr, 10);
			if (*endptr) {
				fprintf(stderr, "Incorrect port value %s\n",
							optarg);
				exit(-1);
			}
			break;
		case 'h':
			host = optarg;
			break;
		default:
			fprintf(stderr, "Unrecognized option %s\n",
						argv[optind]);
			exit(-1);
			break;
		}
	}

	if (!host) {
		fprintf(stderr, "Server address required\n");
		exit(-1);
	}

	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	if (inet_pton(AF_INET, host, &addr.sin_addr) != 1) {
		fprintf(stderr, "Incorrect address %s\n", host);
		exit(-1);
	}

	return client(&addr);
}
