#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <getopt.h>
#include <unistd.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>


static void handle_client(const struct sockaddr_in *addr, int sk)
{
	char addr_str[INET_ADDRSTRLEN];
	char buffer[256];

	printf("Incoming connection from %s:%d\n", inet_ntop(AF_INET,
				&addr->sin_addr, addr_str, INET_ADDRSTRLEN),
				ntohs(addr->sin_port));
	while (1) {
		const int count = read(sk, buffer, sizeof(buffer));

		if (count < 0) {
			perror("Failed to read from client");
			return;
		}

		if (!count) {
			printf("Client closed conection\n");
			break;
		}
		write(1, buffer, count);
	}
}

static int __server(int ssk)
{
	struct sockaddr_in client_addr;
	socklen_t addr_len;

	while (1) {
		const int csk = accept(ssk, (struct sockaddr *)&client_addr,
					&addr_len);

		if (csk < 0) {
			perror("Failed to accept incomming connection");
			return -1;
		}
		handle_client(&client_addr, csk);
		close(csk);
	}
	return 0;
}

static int server(int port)
{
	const int sk = socket(AF_INET, SOCK_STREAM, 0);
	struct sockaddr_in addr;

	if (sk < 0) {
		perror("Failed to create socket");
		return -1;
	}

	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = INADDR_ANY;
	addr.sin_port = htons(port);

	if (bind(sk, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
		close(sk);
		perror("Failed to bind socket to the specified port");
		return -1;
	}

	if (listen(sk, 0) < 0) {
		close(sk);
		perror("Failed to start listening on the soscket");
		return -1;
	}

	const int rc = __server(sk);

	close(sk);
	return rc;
}

int main(int argc, char **argv)
{
	static const struct option long_opts[] = {
		{"port", required_argument, NULL, 'p'},
		{NULL, 0, NULL, 0}
	};
	static const char *optstr = "p:";

	int port = 13131;

	while (1) {
		const int c = getopt_long(argc, argv, optstr, long_opts, NULL);
		char *endptr;

		if (c == -1)
			break;

		switch (c) {
		case 'p':
			port = strtoul(optarg, &endptr, 10);
			if (*endptr) {
				fprintf(stderr, "Incorrect port value %s\n",
							optarg);
				exit(-1);
			}
			break;
		default:
			fprintf(stderr, "Unrecognized option %s\n",
						argv[optind]);
			exit(-1);
			break;
		}
	}

	return server(port);
}
